@@ -1,6 +1,6 @@
 <?php
 /*
-$Id: confmain.php,v 1.129 2011/05/22 15:02:14 gruberroland Exp $
+$Id: confmain.php,v 1.131 2011/08/23 19:04:35 gruberroland Exp $
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
   Copyright (C) 2003 - 2011  Roland Gruber
@@ -314,6 +314,9 @@ if (isLAMProVersion()) {
 	$pwdMailFrom = new htmlTableExtendedInputField(_('From address'), 'pwdResetMail_from', $conf->getLamProMailFrom(), '550');
 	$pwdMailContent->addElement($pwdMailFrom, true);
 	
+	$pwdMailReplyTo = new htmlTableExtendedInputField(_('Reply-to address'), 'pwdResetMail_replyTo', $conf->getLamProMailReplyTo(), '554');
+	$pwdMailContent->addElement($pwdMailReplyTo, true);
+	
 	$pwdMailSubject = new htmlTableExtendedInputField(_('Subject'), 'pwdResetMail_subject', $conf->getLamProMailSubject(), '551');
 	$pwdMailContent->addElement($pwdMailSubject, true);
 	
@@ -355,6 +358,8 @@ $securitySettingsContent->addElement($searchSuffixInput, true);
 $searchFilterInput = new htmlTableExtendedInputField(_("LDAP filter"), 'loginSearchFilter', $conf->getLoginSearchFilter(), '221');
 $searchFilterInput->setRequired(true);
 $securitySettingsContent->addElement($searchFilterInput, true);
+// HTTP authentication
+$securitySettingsContent->addElement(new htmlTableExtendedInputCheckbox('httpAuthentication', ($conf->getHttpAuthentication() == 'true'), _('HTTP authentication'), '223', true), true);
 $securitySettingsContent->addElement(new htmlSpacer(null, '10px'), true);
 // new password
 $password1 = new htmlTableExtendedInputField(_("New password"), 'passwd1', null, '212');
@@ -418,6 +423,9 @@ function checkInput() {
 		if (!$conf->setLamProMailFrom($_POST['pwdResetMail_from'])) {
 			$errors[] = array("ERROR", _("From address for password mails is invalid."), $_POST['pwdResetMail_from']);
 		}
+		if (!$conf->setLamProMailReplyTo($_POST['pwdResetMail_replyTo'])) {
+			$errors[] = array("ERROR", _("Reply-to address for password mails is invalid."), $_POST['pwdResetMail_replyTo']);
+		}
 		$conf->setLamProMailSubject($_POST['pwdResetMail_subject']);
 		if (isset($_POST['pwdResetMail_isHTML']) && ($_POST['pwdResetMail_isHTML'] == 'on')) {
 			$conf->setLamProMailIsHTML('true');
@@ -437,6 +445,12 @@ function checkInput() {
 	$conf->setLoginMethod($_POST['loginMethod']);
 	$conf->setLoginSearchFilter($_POST['loginSearchFilter']);
 	$conf->setLoginSearchSuffix($_POST['loginSearchSuffix']);
+	if (isset($_POST['httpAuthentication']) && ($_POST['httpAuthentication'] == 'on')) {
+		$conf->setHttpAuthentication('true');
+	}
+	else {
+		$conf->setHttpAuthentication('false');
+	}
 	if (!$conf->set_Adminstring(implode(";", $adminTextNew))) {
 		$errors[] = array("ERROR", _("List of admin users is empty or invalid!"));
 	}
