@@ -1,9 +1,9 @@
 <?php
 /*
-$Id: index.php,v 1.11 2010/08/31 18:05:17 gruberroland Exp $
+$Id: index.php,v 1.12 2011/01/14 19:16:54 gruberroland Exp $
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
-  Copyright (C) 2003 - 2010  Roland Gruber
+  Copyright (C) 2003 - 2011  Roland Gruber
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ echo $_SESSION['header'];
 			</tr>
 		</table>
 		<br><br>
-		<fieldset>
+		<fieldset class="ui-corner-all">
 			<legend><b> <?php echo _("LAM configuration"); ?> </b></legend>
 		<TABLE border="0">
 		<?php
