@@ -1,6 +1,6 @@
 <?php
 /*
-$Id: mainlogin.php,v 1.12 2010/02/06 11:50:26 gruberroland Exp $
+$Id: mainlogin.php,v 1.16 2010/10/11 17:52:30 gruberroland Exp $
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
   Copyright (C) 2003 - 2010  Roland Gruber
@@ -70,7 +70,15 @@ echo $_SESSION['header'];
 				echo _("Login");
 			?>
 		</title>
-		<link rel="stylesheet" type="text/css" href="../../style/layout.css">
+	<?php 
+		// include all CSS files
+		$cssDirName = dirname(__FILE__) . '/../../style';
+		$cssDir = dir($cssDirName);
+		while ($cssEntry = $cssDir->read()) {
+			if (substr($cssEntry, strlen($cssEntry) - 4, 4) != '.css') continue;
+			echo "<link rel=\"stylesheet\" type=\"text/css\" href=\"../../style/" . $cssEntry . "\">\n";
+		}
+	?>
 		<link rel="shortcut icon" type="image/x-icon" href="../../graphics/favicon.ico">
 	</head>
 	<body>
@@ -87,15 +95,27 @@ echo $_SESSION['header'];
 			// include all JavaScript files
 			$jsDirName = dirname(__FILE__) . '/../lib';
 			$jsDir = dir($jsDirName);
+			$jsFiles = array();
 			while ($jsEntry = $jsDir->read()) {
 				if (substr($jsEntry, strlen($jsEntry) - 3, 3) != '.js') continue;
+				$jsFiles[] = $jsEntry;
+			}
+			sort($jsFiles);
+			foreach ($jsFiles as $jsEntry) {
 				echo "<script type=\"text/javascript\" src=\"../lib/" . $jsEntry . "\"></script>\n";
 			}
 		?>
-		<p align="center"><a href="http://www.ldap-account-manager.org/" target="_blank">
-			<img src="../../graphics/banner.jpg" border=1 alt="LDAP Account Manager"></a>
-		</p>
-		<hr><br>
+		<table border=0 width="100%" class="lamHeader ui-corner-all">
+			<tr>
+				<td align="left" height="30">
+					<a class="lamHeader" href="http://www.ldap-account-manager.org/" target="new_window">&nbsp;<img src="../../graphics/logo32.png" width=24 height=24 class="align-middle" alt="LDAP Account Manager">&nbsp;&nbsp;LDAP Account Manager</a>
+				</td>
+				<td align="right" height=20>
+					<a href="../login.php"><IMG alt="configuration" src="../../graphics/undo.png">&nbsp;<?php echo _("Back to login") ?></a>
+				</td>
+			</tr>
+		</table>
+		<br>
 		<?php
 			// check if config file is writable
 			if (!$cfgMain->isWritable()) {
@@ -132,9 +152,14 @@ echo $_SESSION['header'];
 				<td style="border-style:none" rowspan="3" width="20"></td>
 				<td style="border-style:none" align="center">
 					<input type="password" name="passwd">
-					<input type="submit" name="submit" value="<?php echo _("Ok"); ?>">
+					<button id="submitButton" class="smallPadding" name="submit"><?php echo _("Ok"); ?></button>
 					&nbsp;
 					<?php printHelpLink(getHelp('', '236'), '236'); ?>
+					<script type="text/javascript" language="javascript">
+					jQuery(document).ready(function() {
+						jQuery('#submitButton').button();
+					});
+					</script>
 				</td>
 				<td style="border-style:none" rowspan="3" width="20"></td>
 			</tr>
@@ -147,12 +172,8 @@ echo $_SESSION['header'];
 		</table>
 		</form>
 
-		<p><br><br><br><br><br></p>
+		<p><br><br></p>
 
-		<!-- back to login page -->
-		<p>
-			<a href="../login.php"> <?php echo _("Back to Login"); ?> </a>
-		</p>
 
 	</body>
 </html>
