@@ -1,6 +1,6 @@
 <?php
 /*
-	$Id: delete.php,v 1.86 2011/05/21 10:58:22 gruberroland Exp $
+	$Id: delete.php,v 1.87 2011/09/27 18:21:37 gruberroland Exp $
 
 	This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
 	Copyright (C) 2003 - 2006  Tilo Lutz
@@ -305,6 +305,10 @@ function getChildCount($dn) {
 */
 function deleteDN($dn) {
 	$errors = array();
+	if (($dn == null) || ($dn == '')) {
+		$errors[] = array('ERROR', _('Entry does not exist'));
+		return $errors;
+	}
 	$sr = @ldap_list($_SESSION['ldap']->server(), $dn, 'objectClass=*', array('dn'), 0, 0, 0, LDAP_DEREF_NEVER);
 	if ($sr) {
 		$entries = ldap_get_entries($_SESSION['ldap']->server(), $sr);
