@@ -1,6 +1,6 @@
 <?php
 /*
-$Id: main_header.php,v 1.60 2011/05/19 16:25:07 gruberroland Exp $
+$Id: main_header.php,v 1.63 2011/10/31 18:43:32 gruberroland Exp $
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
   Copyright (C) 2003 - 2011  Roland Gruber
@@ -40,6 +40,7 @@ include_once($headerPrefix . "../lib/tools.inc");
 
 // HTML header and title
 echo $_SESSION['header'];
+echo "<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"" . $headerPrefix . "../graphics/favicon.ico\">\n";
 echo "<title>LDAP Account Manager</title>\n";
 
 // include all CSS files
@@ -57,7 +58,9 @@ $jsDirName = dirname(__FILE__) . '/lib';
 $jsDir = dir($jsDirName);
 $jsFiles = array();
 while ($jsEntry = $jsDir->read()) {
-	if (substr($jsEntry, strlen($jsEntry) - 3, 3) != '.js') continue;
+	if ((substr($jsEntry, strlen($jsEntry) - 3, 3) != '.js') && (substr($jsEntry, strlen($jsEntry) - 4, 4) != '.php')) {
+		continue;
+	}
 	$jsFiles[] = $jsEntry;
 }
 sort($jsFiles);
@@ -102,6 +105,13 @@ foreach ($toSort as $key => $value) {
 				echo " - " . LAMVersion();
 				?>
 			</a>
+			<?php
+				echo '&nbsp;&nbsp;<small>';
+				$userData = $_SESSION['ldap']->decrypt_login();
+				printf('(' . _('Logged in as: %s') . ')', getAbstractDN($userData[0]));
+				$userData = null;
+				echo '</small>';
+			?>
 		</td>
 	<td align="right" height=30>
 	<ul id="dropmenu" class="dropmenu">
