@@ -1,6 +1,6 @@
 <?php
 /*
-$Id: massBuildAccounts.php,v 1.21 2010/10/24 13:53:44 gruberroland Exp $
+$Id: massBuildAccounts.php,v 1.23 2011/02/13 11:46:52 gruberroland Exp $
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
   Copyright (C) 2004 - 2010  Roland Gruber
@@ -84,10 +84,10 @@ if (isset($_GET['showldif'])) {
 }
 
 include 'main_header.php';
-echo '<div class="userlist-bright smallPaddingContent">';
+echo '<div class="' . $_POST['scope'] . 'list-bright smallPaddingContent">';
 
+$selectedModules = explode(',', $_POST['selectedModules']);
 if ($_FILES['inputfile'] && ($_FILES['inputfile']['size'] > 0)) {
-	$selectedModules = explode(',', $_POST['selectedModules']);
 	// check if input file is well formated
 	$data = array();  // input values without first row
 	$ids = array();  // <column name> => <column number for $data>
@@ -201,14 +201,37 @@ if ($_FILES['inputfile'] && ($_FILES['inputfile']['size'] > 0)) {
 					echo "<a href=\"massBuildAccounts.php?showldif=true\">" . _("Show LDIF file") . "</a>";
 			}
 		}
+		else {
+			massPrintBackButton($_POST['scope'], $selectedModules);
+		}
 	}
 }
 else {
 	StatusMessage('ERROR', _('Please provide a file to upload.'));
-	echo '<br><a href="masscreate.php">' . _('Back') . '</a>';
+	massPrintBackButton($_POST['scope'], $selectedModules);
 }
 
 echo '</div>';
 include 'main_footer.php';
 
+/**
+ * Prints a back button to the page where the user enters a file to upload.
+ *
+ * @param String $scope account type (e.g. user)
+ * @param array $selectedModules selected modules for upload
+ */
+function massPrintBackButton($scope, $selectedModules) {
+	echo '<form enctype="multipart/form-data" action="masscreate.php" method="post">';
+	$container = new htmlTable();
+	$container->addElement(new htmlSpacer(null, '10px'), true);
+	$container->addElement(new htmlButton('submit', _('Back')));
+	$container->addElement(new htmlHiddenInput('type', $scope));
+	for ($i = 0; $i < sizeof($selectedModules); $i++) {
+		$container->addElement(new htmlHiddenInput($scope . '_' . $selectedModules[$i], 'on'));
+	}
+	$tabindex = 1;
+	parseHtml(null, $container, array(), false, $tabindex, $scope);
+	echo '</form>';
+}
+
 ?>
\ No newline at end of file
