@@ -1,6 +1,6 @@
 <?php
 /*
-$Id: masscreate.php,v 1.96 2011/04/04 16:23:05 gruberroland Exp $
+$Id: masscreate.php,v 1.97 2011/05/20 17:43:31 gruberroland Exp $
 
   This code is part of LDAP Account Manager (http://www.ldap-account-manager.org/)
   Copyright (C) 2004 - 2011  Roland Gruber
@@ -361,8 +361,9 @@ function showMainPage($scope, $selectedModules) {
 			$rowCells[] = $columnSpacer;
 			$rowCells[] = new htmlHelpLink($columns[$modules[$m]][$i]['help'], $modules[$m], $scope);
 			$rowCells[] = $columnSpacer;
-			$descriptionText = new htmlOutputText($columns[$modules[$m]][$i]['description'] . $required);
+			$descriptionText = new htmlOutputText($columns[$modules[$m]][$i]['description']);
 			$descriptionText->setMarkAsRequired($required);
+			$descriptionText->setNoWrap(true);
 			$rowCells[] = $descriptionText;
 			$rowCells[] = $columnSpacer;
 			$rowCells[] = new htmlOutputText($columns[$modules[$m]][$i]['name']);
